//+------------------------------------------------------------------+
//|                                               PriceChangeRow.mqh |
//|                                                 Marcin Konieczny |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Marcin Konieczny"

#include <Row.mqh>
//+------------------------------------------------------------------+
//|  CPriceChangeRow                                            |
//+------------------------------------------------------------------+
class CPriceChangeRow : public CRow
  {
private:
   bool              percentChange;
   bool              useArrows;

public:
   //--- 
                     CPriceChangeRow(bool arrows,bool percent=false);

   //---   GetName()  CRow
   virtual string    GetName();

   //---   GetFont()  CRow
   virtual string    GetFont(string symbol,ENUM_TIMEFRAMES tf);

   //---   GetValue(..)  CRow
   virtual string    GetValue(string symbol,ENUM_TIMEFRAMES tf);

   //---   GetColor(..)  CRow
   virtual color     GetColor(string symbol,ENUM_TIMEFRAMES tf);

  };
//+------------------------------------------------------------------+
//|   CPriceChangeRow                               |
//+------------------------------------------------------------------+
CPriceChangeRow::CPriceChangeRow(bool arrows,bool percent=false)
  {
   percentChange=percent;
   useArrows=arrows;
  }
//+------------------------------------------------------------------+
//|   GetName()  CRow                     |
//+------------------------------------------------------------------+
string CPriceChangeRow::GetName()
  {
   return("PriceChg");
  }
//+------------------------------------------------------------------+
//|   GetFont()  CRow                     |
//+------------------------------------------------------------------+
string CPriceChangeRow::GetFont(string symbol,ENUM_TIMEFRAMES tf)
  {
//---    up/down    Wingdings 
   if(useArrows)
      return("Wingdings");
   else
      return("Arial");
  }
//+------------------------------------------------------------------+
//|   GetValue(..)  CRow                  |
//+------------------------------------------------------------------+
string CPriceChangeRow::GetValue(string symbol,ENUM_TIMEFRAMES tf)
  {
   double close[1];
   double open[1];

//---   open  close  
   if(CopyClose(symbol,tf,0, 1, close) < 0) return(" ");
   if(CopyOpen(symbol, tf, 0, 1, open) < 0) return(" ");

//---    
   double change=close[0]-open[0];

   if(useArrows)
     {
      if(change > 0) return(CharToString(233)); //   "up"
      if(change < 0) return(CharToString(234)); //   "down"
      return(" ");
        }else{
      if(percentChange)
        {
         //---    
         return(DoubleToString(change/open[0]*100.0,3)+"%");
        }
      else
        {
         return(DoubleToString(change,(int)SymbolInfoInteger(symbol,SYMBOL_DIGITS)));
        }
     }
  }
//+------------------------------------------------------------------+
//|   GetColor(..)  CRow                  |
//+------------------------------------------------------------------+
color CPriceChangeRow::GetColor(string symbol,ENUM_TIMEFRAMES tf)
  {
   double close[1];
   double open[1];

//---   open  close  
   if(CopyClose(symbol,tf,0, 1, close) < 0) return(clrWhite);
   if(CopyOpen(symbol, tf, 0, 1, open) < 0) return(clrWhite);

   if(close[0] > open[0]) return(clrLime);
   if(close[0] < open[0]) return(clrRed);
   return(clrWhite);
  }
//+------------------------------------------------------------------+

