//+------------------------------------------------------------------+
//|                                                   PriceMARow.mqh |
//|                                                 Marcin Konieczny |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Marcin Konieczny"

#include <Row.mqh>
//+------------------------------------------------------------------+
//| CPriceMARow class                                                |
//+------------------------------------------------------------------+
class CPriceMARow : public CRow
  {
private:
   int               maPeriod;      //  Moving Average
   int               maShift;       //  Moving Average
   ENUM_MA_METHOD    maType;        // SMA, EMA, SMMA  LWMA
   string            symbols[];     //  symbols
   ENUM_TIMEFRAMES   timeframes[];  //  timeframes
   int               handles[];     //   MA

   //---       
   int               GetHandle(string symbol,ENUM_TIMEFRAMES tf);

public:
   //--- 
                     CPriceMARow(ENUM_MA_METHOD type,int period,int shift);

   //---   GetValue(..)  CRow
   virtual string    GetValue(string symbol,ENUM_TIMEFRAMES tf);

   //---   GetName()  CRow
   virtual string    GetName();

   //---   Init(..)  CRow
   virtual void      Init(string &symb[],ENUM_TIMEFRAMES &tfs[]);
  };
//+------------------------------------------------------------------+
//|   CPriceMARow                                   |
//+------------------------------------------------------------------+
CPriceMARow::CPriceMARow(ENUM_MA_METHOD type,int period,int shift)
  {
   maPeriod= period;
   maShift = shift;
   maType=type;
  }
//+------------------------------------------------------------------+
//|   Init(..)  CRow                      |
//+------------------------------------------------------------------+
void CPriceMARow::Init(string &symb[],ENUM_TIMEFRAMES &tfs[])
  {
   int size=ArraySize(symb);

   ArrayResize(symbols,size);
   ArrayResize(timeframes,size);
   ArrayResize(handles,size);

   ArrayInitialize(handles,-1);

//---      
   ArrayCopy(symbols,symb);
   ArrayCopy(timeframes,tfs);

//---    MA    
   for(int i=0; i<ArraySize(symbols); i++)
      handles[i]=iMA(symbols[i],timeframes[i],maPeriod,maShift,maType,PRICE_CLOSE);
  }
//+------------------------------------------------------------------+
//|   GetValue(..)  CRow                  |
//+------------------------------------------------------------------+
string CPriceMARow::GetValue(string symbol,ENUM_TIMEFRAMES tf)
  {
   double value[1];
   MqlTick tick;

//---    MA
   int handle=GetHandle(symbol,tf);

   if(handle==INVALID_HANDLE) return("err");

//---     MA
   if(CopyBuffer(handle,0,0,1,value)<0) return("-");
//---   
   if(!SymbolInfoTick(symbol,tick)) return("-");

//---  : price > MA
   if(tick.bid>value[0])
      return("Yes");
   else
      return("No");
  }
//+------------------------------------------------------------------+
//|   GetName()  CRow                     |
//+------------------------------------------------------------------+
string CPriceMARow::GetName()
  {
   string name;

   switch(maType)
     {
      case MODE_SMA: name = "SMA"; break;
      case MODE_EMA: name = "EMA"; break;
      case MODE_SMMA: name = "SMMA"; break;
      case MODE_LWMA: name = "LWMA"; break;
     }

   return("Price>"+name+"("+IntegerToString(maPeriod)+")");
  }
//+------------------------------------------------------------------+
//|           |
//+------------------------------------------------------------------+
int CPriceMARow::GetHandle(string symbol,ENUM_TIMEFRAMES tf)
  {
   for(int i=0; i<ArraySize(timeframes); i++)
      if(symbols[i]==symbol && timeframes[i]==tf)
         return(handles[i]);

   return(INVALID_HANDLE);
  }
//+------------------------------------------------------------------+
